package in.mobiocean.printer;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Handler;
import android.os.Message;
import android.widget.Toast;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.telpo.tps550.api.TelpoException;
import com.telpo.tps550.api.printer.UsbThermalPrinter;
import com.telpo.tps550.api.util.StringUtil;
import com.telpo.tps550.api.util.SystemUtil;

/** PrinterPlugin */
public class PrinterPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  private UsbThermalPrinter mUsbThermalPrinter;

  private final int NOPAPER = 3;
  private final int PRINTCONTENT = 9;
  private final int LOWPOWER = 10;
  private final int PRINTERR = 11;
  private final int OVERHEAT = 12;

  private MyHandler handler;
  private String printContent = null;

  private Result result;

  private Activity activity;
  private Context context;

  private class MyHandler extends Handler {
    @Override
    public void handleMessage(Message msg) {
      switch (msg.what) {
        case LOWPOWER:
          if (result != null) result.success("101");
          break;
        case NOPAPER:
          if (result != null) result.success("102");
          break;
        case PRINTCONTENT:
          new contentPrintThread().start();
          break;
        case OVERHEAT:
          if (result != null) result.success("103");
          break;
        default:
          if (result != null) result.success("104");
          break;
      }
    }
  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "printer");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding){
    activity = binding.getActivity();
    context = binding.getActivity();
    mUsbThermalPrinter = new UsbThermalPrinter(context);
    handler = new MyHandler();
  }

  @Override
  public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding){
    activity = binding.getActivity();
    context = binding.getActivity();
    mUsbThermalPrinter = new UsbThermalPrinter(context);
    handler = new MyHandler();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if (call.method.equals("printContent")) {
      this.result = result;
      printContent = call.argument("content");
      handler.sendMessage(handler.obtainMessage(PRINTCONTENT, 1, 0, null));
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onDetachedFromActivity() {
    context = null;
    activity = null;
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
    context = null;
    activity = null;
  }

  private class contentPrintThread extends Thread {
    @Override
    public void run() {
      super.run();
      try {
        mUsbThermalPrinter.start(1);
        mUsbThermalPrinter.reset();
        mUsbThermalPrinter.setMonoSpace(true);
        mUsbThermalPrinter.setAlgin(UsbThermalPrinter.ALGIN_MIDDLE);
        mUsbThermalPrinter.setLeftIndent(0);
        mUsbThermalPrinter.setLineSpace(0);
        mUsbThermalPrinter.setTextSize(20);
        mUsbThermalPrinter.setGray(7);
        mUsbThermalPrinter.addString(printContent);
        mUsbThermalPrinter.printString();
        mUsbThermalPrinter.walkPaper(10);
        mUsbThermalPrinter.reset();
        if (result != null)
          result.success("100");
      } catch (TelpoException e) {
        e.printStackTrace();
        String Result = e.toString();
        if (Result.equals("com.telpo.tps550.api.printer.NoPaperException")) {
          handler.sendMessage(handler.obtainMessage(NOPAPER, 1, 0, null));
        } else if (Result.equals("com.telpo.tps550.api.printer.OverHeatException")) {
          handler.sendMessage(handler.obtainMessage(OVERHEAT, 1, 0, null));
        } else if (Result.equals("com.telpo.tps550.api.printer.LowPowerException")) {
          handler.sendMessage(handler.obtainMessage(LOWPOWER, 1, 0, null));
        } else {
          handler.sendMessage(handler.obtainMessage(PRINTERR, 1, 0, null));
        }
      }
    }
  }
}
